@@ -2,19 +2,19 @@
 
 """
 peakrdl-python is a tool to generate Python Register Access Layer (RAL) from SystemRDL
-Copyright (C) 2021 - 2023
+Copyright (C) 2021 - 2025
 
 This program is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
+it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+GNU Lesser General Public License for more details.
 
-You should have received a copy of the GNU General Public License
+You should have received a copy of the GNU Lesser General Public License
 along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 script to generate all the test cases from the test set
@@ -26,11 +26,13 @@ from glob import glob
 from typing import Optional, List
 import argparse
 import pathlib
+from itertools import product
 
 from systemrdl import RDLCompiler # type: ignore
 from systemrdl.node import Node, AddrmapNode # type: ignore
 from peakrdl_ipxact import IPXACTImporter # type: ignore
 from src.peakrdl_python import PythonExporter # type: ignore
+from src.peakrdl_python import NodeHashingMethod # type: ignore
 
 test_case_path = os.path.join('tests', 'testcases')
 
@@ -40,6 +42,22 @@ CommandLineParser.add_argument('--output', dest='output_path',
                                default='testcase_output')
 CommandLineParser.add_argument('--test_case', dest='test_case',
                                type=str)
+CommandLineParser.add_argument('--copy_libraries', action='store_true', dest='copy_libraries',
+                               help='by default peakrdl python copies all the libraries over'
+                                    'to the generated package along with the generated code. '
+                                    'However, that is potentiality problematic when developing'
+                                    'and debugging as multiple copies of the libraries can cause'
+                                    'confusion. Therefore by default this script does not copy '
+                                    'them over.')
+CommandLineParser.add_argument('--hashing_mode',
+                               dest='hashing_mode',
+                               type=str,
+                               choices=[item.name for item in NodeHashingMethod],
+                               default='PYTHONHASH',
+                               help='The method used to generate the hash of the node, in order to '
+                                    'deduplicate the register model. Set this to `SHA256` if '
+                                    'the python names need to stay consistent one export to the '
+                                    'next. However, this mode is slower')
 
 
 def compile_rdl(infile: str,
@@ -75,17 +93,17 @@ def compile_rdl(infile: str,
 
 def generate(root: Node, outdir: str,
              asyncoutput: bool = False,
-             skip_test_case_generation: bool = False,
              legacy_block_access: bool = True,
-             legacy_enum_type: bool = True) -> List[str]:
+             legacy_enum_type: bool = True,
+             copy_library: bool = False,
+             skip_systemrdl_name_and_desc_in_docstring: bool = False,
+             hashing_mode: NodeHashingMethod = NodeHashingMethod.PYTHONHASH) -> List[str]:
     """
     Generate a PeakRDL output package from compiled systemRDL
 
     Args:
         root: node in the systemRDL from which the code should be generated
         outdir: directory to store the result in
-        autoformatoutputs: If set to True the code will be run through autopep8 to
-                clean it up. This can slow down large jobs or mask problems
         legacy_block_access: If set to True the code build a register model the legacy array block
                              access as opposed to the newer list based
         legacy_enum_type: If set to True the code with build with the old style IntEnum rather than
@@ -96,11 +114,15 @@ def generate(root: Node, outdir: str,
 
     """
     print(f'Info: Generating python for {root.inst_name} in {outdir}')
-    modules = PythonExporter().export(root, outdir, # type: ignore[no-untyped-call]
-                                      asyncoutput=asyncoutput,
-                                      skip_test_case_generation=skip_test_case_generation,
-                                      legacy_block_access=legacy_block_access,
-                                      legacy_enum_type=legacy_enum_type)
+    modules = PythonExporter().export(
+        root, outdir,  # type: ignore[no-untyped-call]
+        asyncoutput=asyncoutput,
+        legacy_block_access=legacy_block_access,
+        legacy_enum_type=legacy_enum_type,
+        skip_library_copy=not copy_library,
+        skip_systemrdl_name_and_desc_in_docstring=
+        skip_systemrdl_name_and_desc_in_docstring,
+        hashing_method=hashing_mode)
 
     return modules
 
@@ -133,28 +155,38 @@ if __name__ == '__main__':
         else:
             root = compile_rdl(rdl_file)
 
-        for build_options, folder_name in \
-                [({'asyncoutput': True, 'legacy_block':False, 'legacy_enum': False}, 'raw_async'),
-                 ({'asyncoutput': False, 'legacy_block':False, 'legacy_enum': False}, 'raw'),
-                 ({'asyncoutput': True, 'legacy_block': True, 'legacy_enum': False}, 'raw_async_legacy_block'),
-                 ({'asyncoutput': False, 'legacy_block': True, 'legacy_enum': False}, 'raw_legacy_block'),
-                 ({'asyncoutput': True, 'legacy_block': False, 'legacy_enum': True}, 'raw_async_legacy_enum'),
-                 ({'asyncoutput': False, 'legacy_block': False, 'legacy_enum': True}, 'raw_legacy_enum'),
-                 ({'asyncoutput': True, 'legacy_block': True, 'legacy_enum': True}, 'raw_async_legacy_block_legacy_enum'),
-                 ({'asyncoutput': False, 'legacy_block': True, 'legacy_enum': True}, 'raw_legacy_block_legacy_enum')
-                 ]:
+        options = {
+            'asyncoutput': [True, False],
+            'legacy': [True, False],
+            'skip_systemrdl_name_and_desc_in_docstring': [True, False]
+        }
+
+        for asyncoutput, legacy, skip_name_and_desc_in_docstring in product(
+                options['asyncoutput'], options['legacy'],
+                options['skip_systemrdl_name_and_desc_in_docstring'] ):
 
             # test cases that use the extended widths an not be tested in the non-legacy modes
             if (testcase_name in ['extended_memories', 'extended_sizes_registers_array']) and \
-                    (build_options['legacy_block'] is True):
+                    (legacy is True):
                 continue
 
-            _ = generate(root, str(output_path / folder_name),
-                         asyncoutput=build_options['asyncoutput'],
-                         legacy_block_access=build_options['legacy_block'],
-                         legacy_enum_type=build_options['legacy_enum'])
+            folder_parts = 'raw'
+            if asyncoutput:
+                folder_parts += '_async'
+            if legacy:
+                folder_parts += '_legacy'
+            if skip_name_and_desc_in_docstring:
+                folder_parts += '_skip_name_and_desc_in_docstring'
 
-            module_fqfn = output_path / folder_name / '__init__.py'
+            _ = generate(root, str(output_path / folder_parts),
+                         asyncoutput=asyncoutput,
+                         legacy_block_access=legacy,
+                         legacy_enum_type=legacy,
+                         copy_library=CommandLineArgs.copy_libraries,
+                         skip_systemrdl_name_and_desc_in_docstring=skip_name_and_desc_in_docstring,
+                         hashing_mode=NodeHashingMethod[CommandLineArgs.hashing_mode])
+
+            module_fqfn = output_path / folder_parts / '__init__.py'
             with open(module_fqfn, 'w', encoding='utf-8') as fid:
                 fid.write('pass\n')
 
