@@ -1,40 +1,40 @@
 """
 peakrdl-python is a tool to generate Python Register Access Layer (RAL) from SystemRDL
-Copyright (C) 2021 - 2023
+Copyright (C) 2021 - 2025
 
 This program is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
+it under the terms of the GNU Lesser General Public License as 
+published by the Free Software Foundation, either version 3 of 
+the License, or (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+GNU Lesser General Public License for more details.
 
-You should have received a copy of the GNU General Public License
+You should have received a copy of the GNU Lesser General Public License
 along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 A set of utility functions that perform supplementary processing on a node in a compiled
 system RDL dataset.
 """
-from typing import Iterable, Optional, List, Protocol
+from typing import Optional, Protocol
+from collections.abc import Iterable
 from itertools import filterfalse
 
 import textwrap
 
-from systemrdl.node import Node  # type: ignore
-from systemrdl.node import RegNode  # type: ignore
-from systemrdl.node import AddressableNode  # type: ignore
-from systemrdl.node import FieldNode  # type: ignore
-from systemrdl.node import MemNode  # type: ignore
-from systemrdl.node import SignalNode  # type: ignore
-from systemrdl.node import AddrmapNode  # type: ignore
-from systemrdl.node import RegfileNode  # type: ignore
-from systemrdl.component import Component  # type: ignore
-from systemrdl.rdltypes.user_enum import UserEnumMeta  # type: ignore
-from systemrdl import RDLListener, WalkerAction, RDLWalker  # type: ignore
+from systemrdl.node import Node
+from systemrdl.node import RegNode
 
+from systemrdl.node import AddressableNode
+from systemrdl.node import FieldNode
+from systemrdl.node import MemNode
+from systemrdl.node import SignalNode
+
+from systemrdl.rdltypes.user_enum import UserEnumMeta
+
+from .lib.utility_functions import calculate_bitmask
 
 class HideNodeCallback(Protocol):
     """
@@ -45,25 +45,14 @@ class HideNodeCallback(Protocol):
     def __call__(self, node: Node) -> bool:
         pass
 
-
-def get_fully_qualified_type_name(node: Node) -> str:
+class ShowUDPCallback(Protocol):
     """
-    Returns the fully qualified class type name, i.e. with scope prefix
+    Callback function that determines whether a UDP on a node shown, this is intended
+    to be used with the RegEX check or list of names
     """
-    scope_path = node.inst.get_scope_path(scope_separator='_')
-
-    # TODO if the node is a integer field we can use the base class no
-    #      need to generate a unique instance, if it has not documentation
-    #      properties
-
-    inst_type_name = node.inst.type_name
-    if inst_type_name is None:
-        inst_type_name = node.inst_name
-
-    if (scope_path == '') or (scope_path is None):
-        return inst_type_name
-
-    return scope_path + '_' + inst_type_name
+    # pylint: disable=too-few-public-methods
+    def __call__(self, udp_name: str) -> bool:
+        pass
 
 
 def hide_based_on_property(node: Node, show_hidden: bool) -> bool:
@@ -72,7 +61,7 @@ def hide_based_on_property(node: Node, show_hidden: bool) -> bool:
 
     Args:
         node: a system RDL node
-        show_hidden: a boolean to indicate that the property should be ingored
+        show_hidden: a boolean to indicate that the property should be ignored
 
     Returns:
         True if the node should be hidden
@@ -80,97 +69,7 @@ def hide_based_on_property(node: Node, show_hidden: bool) -> bool:
     return node.get_property('python_hide', default=False) and not show_hidden
 
 
-def get_dependent_component(node: AddressableNode,
-                            hide_node_callback: HideNodeCallback) -> Iterable[Node]:
-    """
-    iterable of nodes that have a component which is used by a
-    descendant, this list is de-duplicated and reversed to components
-    are declared before their parents who use them
-
-    Args:
-        node: node to be analysed
-        hide_node_callback: callback to determine if the node should be hidden
-
-
-    """
-    class UniqueComponents(RDLListener):
-        """
-        class intended to be used as part of the walker/listener protocol to find all the items
-        non-hidden nodes
-        """
-
-        def __init__(self, hide_node_callback: HideNodeCallback) -> None:
-            super().__init__()
-
-            self.__hide_node_callback = hide_node_callback
-            self.__components_needed: list[Component] = []
-            self.nodes: list[Node] = []
-
-
-        def enter_Reg(self, node: RegNode) -> Optional[WalkerAction]:
-            if self.__hide_node_callback(node):
-                return WalkerAction.SkipDescendants
-
-            if node.inst in self.__components_needed:
-                return WalkerAction.SkipDescendants
-
-            self.__components_needed.append(node.inst)
-            self.nodes.append(node)
-            return WalkerAction.Continue
-
-        def enter_Mem(self, node: MemNode) -> Optional[WalkerAction]:
-            if self.__hide_node_callback(node):
-                return WalkerAction.SkipDescendants
-
-            if node.inst in self.__components_needed:
-                return WalkerAction.SkipDescendants
-
-            self.__components_needed.append(node.inst)
-            self.nodes.append(node)
-            return WalkerAction.Continue
-
-        def enter_Field(self, node: FieldNode) -> Optional[WalkerAction]:
-            if self.__hide_node_callback(node):
-                return WalkerAction.SkipDescendants
-
-            if node.inst in self.__components_needed:
-                return WalkerAction.SkipDescendants
-
-            self.__components_needed.append(node.inst)
-            self.nodes.append(node)
-            return WalkerAction.Continue
-
-        def enter_Addrmap(self, node: AddrmapNode) -> Optional[WalkerAction]:
-            if self.__hide_node_callback(node):
-                return WalkerAction.SkipDescendants
-
-            if node.inst in self.__components_needed:
-                return WalkerAction.SkipDescendants
-
-            self.__components_needed.append(node.inst)
-            self.nodes.append(node)
-            return WalkerAction.Continue
-
-        def enter_Regfile(self, node: RegfileNode) -> Optional[WalkerAction]:
-            if self.__hide_node_callback(node):
-                return WalkerAction.SkipDescendants
-
-            if node.inst in self.__components_needed:
-                return WalkerAction.SkipDescendants
-
-            self.__components_needed.append(node.inst)
-            self.nodes.append(node)
-            return WalkerAction.Continue
-
-    unique_component_walker = UniqueComponents(hide_node_callback=hide_node_callback)
-    # running the walker populated the blocks with all the address maps in within the
-    # top block, including the top_block itself
-    RDLWalker(unroll=True).walk(node, unique_component_walker, skip_top=False)
-
-    return reversed(unique_component_walker.nodes)
-
-
-def get_table_block(node: Node) -> str:
+def get_table_block(  node: Node, skip_systemrdl_name_and_desc_in_docstring:bool) -> str:
     """
     Converts the documentation for a systemRDL node into a nicely formatted table that can be
     inserted into the docstring of the class
@@ -181,6 +80,9 @@ def get_table_block(node: Node) -> str:
     Returns:
         A string that represents a sphinx table
     """
+    if skip_systemrdl_name_and_desc_in_docstring:
+        return ''
+
     row_break = '+--------------+' \
                 '-------------------------------------------------------------------------+'
     if ('name' in node.list_properties()) or ('desc' in node.list_properties()):
@@ -194,7 +96,10 @@ def get_table_block(node: Node) -> str:
         if 'name' in node.list_properties():
             table_strs.append("| Name         | .. raw:: html".ljust(88, ' ') + ' |')
             table_strs.append("|              | ".ljust(88, ' ') + ' |')
-            name_rows = textwrap.wrap(node.get_html_name(), width=88,
+            html_name = node.get_html_name()
+            if html_name is None:
+                raise ValueError('html name should not be None')
+            name_rows = textwrap.wrap(html_name, width=88,
                                       initial_indent="|              |      ",
                                       subsequent_indent="|              |      ")
             for name_row in name_rows:
@@ -204,7 +109,10 @@ def get_table_block(node: Node) -> str:
         if 'desc' in node.list_properties():
             table_strs.append("| Description  | .. raw:: html".ljust(88, ' ') + ' |')
             table_strs.append("|              | ".ljust(88, ' ') + ' |')
-            desc_rows = textwrap.wrap(node.get_html_desc(), width=88,
+            html_desc = node.get_html_desc()
+            if html_desc is None:
+                raise ValueError('html name should not be None')
+            desc_rows = textwrap.wrap(html_desc, width=88,
                                       initial_indent="|              |      ",
                                       subsequent_indent="|              |      ")
             for desc_row in desc_rows:
@@ -233,7 +141,7 @@ def get_field_bitmask_int(node: FieldNode) -> int:
     if not isinstance(node, FieldNode):
         raise TypeError(f'node is not a {type(FieldNode)} got {type(node)}')
 
-    return sum(2 ** x for x in range(node.low, node.high + 1))
+    return calculate_bitmask(high=node.high, low=node.low)
 
 
 def get_field_bitmask_hex_string(node: FieldNode) -> str:
@@ -266,6 +174,8 @@ def get_field_inv_bitmask_hex_string(node: FieldNode) -> str:
     """
     if not isinstance(node, FieldNode):
         raise TypeError(f'node is not a {type(FieldNode)} got {type(node)}')
+    if not isinstance(node.parent, RegNode):
+        raise TypeError(f'node.parent is not a {type(RegNode)} got {type(node)}')
     reg_bitmask = (2 ** (node.parent.size * 8)) - 1
     inv_bitmask = reg_bitmask ^ get_field_bitmask_int(node)
     return f'0x{inv_bitmask:X}'
@@ -338,7 +248,7 @@ def get_reg_fields(node: RegNode, hide_node_callback: HideNodeCallback) -> Itera
 
     """
     if not isinstance(node, RegNode):
-        raise TypeError(f'node is not a {type(RegNode)} got {type(node)}')
+        raise TypeError(f'node is not a RegNode got {type(node)}')
 
     return filterfalse(hide_node_callback, node.fields())
 
@@ -382,6 +292,42 @@ def get_reg_readable_fields(node: RegNode,
     return filter(lambda x: x.is_sw_readable,
                   get_reg_fields(node=node, hide_node_callback=hide_node_callback))
 
+def get_reg_accesswidth(node: RegNode) -> int:
+    """
+    Determine register access width in bits, with a default based on the size of the register
+
+    Args:
+        node: node to be analysed
+
+    Returns:
+        register access width in bits
+
+    """
+    if not isinstance(node, RegNode):
+        raise TypeError(f'node is not a {type(RegNode)} got {type(node)}')
+
+    if 'accesswidth' in node.list_properties():
+        return node.get_property('accesswidth')
+    return node.size*8
+
+def get_reg_regwidth(node: RegNode) -> int:
+    """
+    Determine register regwidth in bits, with a default based on the size of the register
+
+    Args:
+        node: node to be analysed
+
+    Returns:
+        register regwidth in bits
+
+    """
+    if not isinstance(node, RegNode):
+        raise TypeError(f'node is not a {type(RegNode)} got {type(node)}')
+
+    if 'regwidth' in node.list_properties():
+        return node.get_property('regwidth')
+    return node.size*8
+
 
 def uses_memory(node: AddressableNode) -> bool:
     """
@@ -436,6 +382,24 @@ def get_memory_width_bytes(node: MemNode) -> int:
 
     return node.size // node.get_property('mementries')
 
+def get_memory_accesswidth(node: MemNode) -> int:
+    """
+    Determine register access width in bits, with a default based on the size of the register
+
+    Args:
+        node: node to be analysed
+
+    Returns:
+        register access width in bits
+
+    """
+    if not isinstance(node, MemNode):
+        raise TypeError(f'node is not a {type(MemNode)} got {type(node)}')
+
+    if 'accesswidth' in node.list_properties():
+        return node.get_property('accesswidth')
+    return node.get_property('memwidth')
+
 
 def get_field_default_value(node: FieldNode) -> Optional[int]:
     """
@@ -463,7 +427,7 @@ def get_field_default_value(node: FieldNode) -> Optional[int]:
     raise TypeError(f'unhandled type for field default type={type(value)}')
 
 
-def get_enum_values(enum: UserEnumMeta) -> List[int]:
+def get_enum_values(enum: UserEnumMeta) -> list[int]:
     """
 
     Args:
@@ -478,18 +442,41 @@ def get_enum_values(enum: UserEnumMeta) -> List[int]:
     return [e.value for e in enum]
 
 
-def get_properties_to_include(node: Node, udp_to_include: Optional[List[str]]) -> List[str]:
+def get_properties_to_include(node: Node, udp_to_include_callback: ShowUDPCallback) -> list[str]:
     """
-
+    Provide a list of the User Defined Properties to include in the Register Model for a given
+    Node, based on a call back
 
     Args:
         node: the system rdl node to examine the properties of
-        udp_to_include: list of property names to include in the system rdl
+        udp_to_include_callback: a function that determines whether to include a UDP ot not
 
     Returns:
         list of properties
     """
-    if udp_to_include is None:
-        return []
     nodal_properties = node.list_properties(include_udp=True, include_native=False)
-    return list(filter(lambda x: x in udp_to_include, nodal_properties))
+    return list(filter(udp_to_include_callback, nodal_properties))
+
+def is_encoded_field(node: FieldNode) -> bool:
+    """
+    Determines if a field node is encoded (using an enumerated value)
+    Args:
+        node:
+
+    Returns: True if encoded
+    """
+    if not isinstance(node, FieldNode):
+        raise TypeError(f'This should only be called on an FieldNode, got {type(node)}')
+    return 'encode' in node.list_properties()
+
+def full_slice_accessor(node: AddressableNode) -> str:
+    """
+    Provides a node full access slice, for example if the node has 3 dimensions, this function
+    return "[:,:,:]"
+    """
+    if not node.is_array:
+        raise RuntimeError('node is expected to be an array')
+    dimensions = node.array_dimensions
+    if dimensions is None:
+        raise RuntimeError('array node should have dimensions')
+    return '[' + ','.join([':' for _ in range(len(dimensions))]) + ']'
