@@ -1,4 +1,20 @@
 """
+peakrdl-python is a tool to generate Python Register Access Layer (RAL) from SystemRDL
+Copyright (C) 2021 - 2025
+
+This program is free software: you can redistribute it and/or modify
+it under the terms of the GNU Lesser General Public License as 
+published by the Free Software Foundation, either version 3 of 
+the License, or (at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU Lesser General Public License for more details.
+
+You should have received a copy of the GNU Lesser General Public License
+along with this program.  If not, see <https://www.gnu.org/licenses/>.
+
 Test exporting this tests a couple of different things:
 1. calling the exporter
 2. tests some of the export options that would not other be checked with the integration tests get
@@ -8,11 +24,12 @@ import unittest
 import os
 import tempfile
 import sys
-from typing import List
 import re
 from itertools import chain, permutations, product
 from pathlib import Path
 from array import array as Array
+import inspect
+from typing import Optional
 
 from contextlib import contextmanager
 
@@ -25,6 +42,10 @@ from peakrdl_python import compiler_with_udp_registers
 from peakrdl_python.__about__ import __version__ as peakrdl_version
 from peakrdl_python.__peakrdl__ import Exporter as PeakRDLPythonExported
 from peakrdl_python.lib.utility_functions import get_array_typecode
+from peakrdl_python.lib import Node, FieldReadWrite, NodeArray
+from peakrdl_python.lib import NormalCallbackSet
+from peakrdl_python.sim_lib.dummy_callbacks import dummy_read
+
 
 if sys.version_info[0:2] < (3, 11):
     # Prior to py3.11, tomllib is a 3rd party package
@@ -46,7 +67,6 @@ class TestExportHidden(unittest.TestCase):
     test_case_path = test_cases
     test_case_name = 'hidden_property.rdl'
     test_case_top_level = 'hidden_property'
-    test_case_reg_model_cls = test_case_top_level + '_cls'
 
     @contextmanager
     def build_python_wrappers_and_make_instance(self, show_hidden):
@@ -87,12 +107,12 @@ class TestExportHidden(unittest.TestCase):
             # add the temp directory to the python path so that it can be imported from
             sys.path.append(tmpdirname)
 
-            reg_model_module = __import__( temp_package_name + '.' + self.test_case_top_level +
-                '.reg_model.' + self.test_case_top_level,
-                globals(), locals(), [self.test_case_reg_model_cls], 0)
-            dut_cls = getattr(reg_model_module, self.test_case_reg_model_cls)
-            peakrdl_python_package = __import__(temp_package_name + '.' +
-                                                self.test_case_top_level + '.lib',
+            reg_model_module = __import__(
+                temp_package_name + '.' + self.test_case_top_level + '.reg_model',
+                globals(), locals(), ['RegModel'], 0)
+            dut_cls = getattr(reg_model_module, 'RegModel')
+            peakrdl_python_package = __import__(
+                temp_package_name + '.' + self.test_case_top_level + '.lib',
                 globals(), locals(), ['CallbackSet'], 0)
             callbackset_cls = getattr(peakrdl_python_package, 'NormalCallbackSet')
             dummy_operations_module = __import__(temp_package_name + '.' +
@@ -139,10 +159,12 @@ class TestExportUDP(unittest.TestCase):
     test_case_reg_model_cls = test_case_top_level + '_cls'
 
     @contextmanager
-    def build_wrappers_and_import(self, udp_list:List[str]):
+    def build_wrappers_and_import(self,
+                                  udp_list:Optional[list[str]]=None,
+                                  udp_regex:Optional[str]=None):
         """
-        Context manager to build the python wrappers for a value of show_hidden, then import them
-        and clean up afterwards
+        Context manager to build the python wrappers with the specified UDPs included then import
+        them and clean up afterwards
         """
 
         # compile the code for the test
@@ -156,7 +178,12 @@ class TestExportUDP(unittest.TestCase):
             # the temporary package, within which the real package is placed is needed to ensure
             # that there are two separate entries in the python import cache and this avoids the
             # test failing for strange reasons
-            temp_package_name = 'dir_' + str(hash('_'.join(udp_list)))
+            dir_suffix = ''
+            if udp_list is not None:
+                dir_suffix += '_'.join(udp_list)
+            if udp_regex is not None:
+                dir_suffix += udp_regex
+            temp_package_name = 'dir_' + str(hash(dir_suffix))
             fq_package_path = os.path.join(tmpdirname, temp_package_name)
             os.makedirs(fq_package_path)
             with open(os.path.join(fq_package_path, '__init__.py'), 'w', encoding='utf-8') as fid:
@@ -166,10 +193,11 @@ class TestExportUDP(unittest.TestCase):
                             path=fq_package_path,
                             asyncoutput=False,
                             delete_existing_package_content=False,
-                            skip_library_copy=False,
                             skip_test_case_generation=True,
                             legacy_block_access=False,
-                            user_defined_properties_to_include=udp_list)
+                            skip_library_copy=True,
+                            user_defined_properties_to_include=udp_list,
+                            user_defined_properties_to_include_regex=udp_regex)
 
             # add the temp directory to the python path so that it can be imported from
             sys.path.append(tmpdirname)
@@ -178,19 +206,10 @@ class TestExportUDP(unittest.TestCase):
                 '.reg_model.' + self.test_case_top_level,
                 globals(), locals(), [self.test_case_reg_model_cls], 0)
             dut_cls = getattr(reg_model_module, self.test_case_reg_model_cls)
-            peakrdl_python_package = __import__(temp_package_name + '.' +
-                                                self.test_case_top_level + '.lib',
-                globals(), locals(), ['CallbackSet'], 0)
-            callbackset_cls = getattr(peakrdl_python_package, 'NormalCallbackSet')
-            dummy_operations_module = __import__(temp_package_name + '.' +
-                                                 self.test_case_top_level +
-                                                 '.sim_lib.dummy_callbacks',
-                                    globals(), locals(), ['dummy_read', 'dummy_write'], 0)
-            dummy_read = getattr(dummy_operations_module, 'dummy_read')
 
             # no read/write are attempted so this can yield out a version with no callbacks
             # configured
-            yield dut_cls(callbacks=callbackset_cls(read_callback=dummy_read))
+            yield dut_cls(callbacks=NormalCallbackSet(read_callback=dummy_read))
 
             sys.path.remove(tmpdirname)
 
@@ -238,17 +257,32 @@ class TestExportUDP(unittest.TestCase):
                               'enum_property_to_include',
                               'int_property_to_include',
                               'str_property_to_include']
-        for udp_to_include in chain.from_iterable(
-                [permutations(full_property_list, r) for r in range(len(full_property_list))]):
-            with self.subTest(udp_to_include=udp_to_include), \
-                    self.build_wrappers_and_import(udp_list=list(udp_to_include)) as dut:
-                for udp in full_property_list:
-                    if udp in list(udp_to_include):
+        def check_udp_present(dut, udp_to_include:list[str]) -> None:
+            for udp in full_property_list:
+                if udp in udp_to_include:
+                    if udp in udp_to_include:
                         self.assertIn(udp, dut.reg_a.field_a.udp)
+                        self.assertIn(udp, dut.mem_with_properties.udp)
+                        self.assertIn(udp, dut.addrmap_a.udp)
                     else:
                         self.assertNotIn(udp, dut.reg_a.field_a.udp)
+                        self.assertNotIn(udp, dut.mem_with_properties.udp)
+                        self.assertNotIn(udp, dut.addrmap_a.udp)
+            self.assertNotIn('int_property_to_exclude', dut.reg_a.field_a.udp)
 
-                self.assertNotIn('int_property_to_exclude', dut.reg_a.field_a.udp)
+
+        for udp_to_include in chain.from_iterable(
+                [permutations(full_property_list, r) for r in range(len(full_property_list))]):
+            # check the list method
+            with self.subTest(udp_to_include=udp_to_include), \
+                    self.build_wrappers_and_import(udp_list=list(udp_to_include)) as dut:
+                check_udp_present(dut=dut, udp_to_include=list(udp_to_include))
+
+            # check the regex method
+            udp_regex = '|'.join(udp_to_include)
+            with self.subTest(udp_regex=udp_regex), \
+                    self.build_wrappers_and_import(udp_regex=udp_regex) as dut:
+                check_udp_present(dut=dut, udp_to_include=list(udp_to_include))
 
 
 class TestRegexExportHidden(unittest.TestCase):
@@ -423,10 +457,10 @@ class TestUDPDeclarations(unittest.TestCase):
 
     def test_bad_property_component_declaration(self):
         """
-        Test that the property not being defined as a string causes a problem
+        Test that the property with the correct components causes an error
         """
         system_rdl_code = \
-            'property python_inst_name { type = number; component = signal | reg; };' +\
+            'property python_inst_name { type = string; component = signal | reg; };' +\
             'addrmap name_of_addrmap { ' +\
             'reg {' +\
             '    python_inst_name="overidden_reg_a";'+\
@@ -444,7 +478,10 @@ class TestUDPDeclarations(unittest.TestCase):
         Test that the property setting to a python keyname causes an error
         """
         system_rdl_code = \
-            'property python_inst_name { type = number; component = signal | reg; };' +\
+            'property python_inst_name { ' + \
+            '    type = string; ' + \
+            '    component = addrmap | regfile | reg | field | mem; ' + \
+            '};' + \
             'addrmap name_of_addrmap { ' +\
             'reg {' +\
             '    python_inst_name="in";'+\
@@ -723,6 +760,170 @@ class TestCallbackAndLegacyTemplates(unittest.TestCase):
                 else:
                     dut.simple_memory_a.write(0, [0, 0, 0, 0])
 
+class TestNameDescInDocString(unittest.TestCase):
+    """
+    Test class for the inclusion of the name and desc system rdl parameters in the docstring.
+    without them there is greater opportunities for deduplication. In order for this test to
+    work it is necessary to also skip the name and description properties
+    """
+
+    test_case_path = test_cases
+    test_case_name = 'name_desc_option_deduplicate.rdl'
+    test_case_top_level = 'name_desc_option_deduplicate'
+
+    @contextmanager
+    def build_python_wrappers_and_make_instance(self, skip_name_desc_in_docstring):
+        """
+        Context manager to build the python wrappers for a value of show_hidden, then import them
+        and clean up afterwards
+        """
+
+        # compile the code for the test
+        rdlc = compiler_with_udp_registers()
+        rdlc.compile_file(os.path.join(self.test_case_path, self.test_case_name))
+        spec = rdlc.elaborate(top_def_name=self.test_case_top_level).top
+
+        exporter = PythonExporter()
+
+        with tempfile.TemporaryDirectory() as tmpdirname:
+            # the temporary package, within which the real package is placed is needed to ensure
+            # that there are two separate entries in the python import cache and this avoids the
+            # test failing for strange reasons
+            if skip_name_desc_in_docstring:
+                temp_package_name = 'name_desc_excluded'
+            else:
+                temp_package_name = 'name_desc_included'
+            fq_package_path = os.path.join(tmpdirname, temp_package_name)
+            os.makedirs(fq_package_path)
+            with open(os.path.join(fq_package_path, '__init__.py'), 'w', encoding='utf-8') as fid:
+                fid.write('pass\n')
+
+            exporter.export(node=spec,
+                            path=fq_package_path,
+                            asyncoutput=False,
+                            delete_existing_package_content=False,
+                            skip_test_case_generation=True,
+                            skip_library_copy=True,
+                            legacy_block_access=False,
+                            skip_systemrdl_name_and_desc_properties=True,
+                            skip_systemrdl_name_and_desc_in_docstring=skip_name_desc_in_docstring)
+
+            # add the temp directory to the python path so that it can be imported from
+            sys.path.append(tmpdirname)
+
+            reg_model_module = __import__(
+                temp_package_name + '.' + self.test_case_top_level + '.reg_model',
+                globals(), locals(), ['RegModel'], 0)
+            dut_cls = getattr(reg_model_module, 'RegModel')
+
+            # no read/write are attempted so this can yield out a version with no callbacks
+            # configured
+            yield dut_cls(callbacks=NormalCallbackSet(read_callback=dummy_read))
+
+            sys.path.remove(tmpdirname)
+
+    def test_included(self):
+        """
+        Test to make sure all the nodes have a name and description doc string as expected
+        """
+        def check_item(node: Node, force_alt: bool=False):
+            doc_string = inspect.getdoc(node)
+            if node.inst_name.endswith('[0]') or node.inst_name.endswith('[1]'):
+                reduced_inst_name = node.inst_name[:-3]
+            else:
+                reduced_inst_name = node.inst_name
+            if node.inst_name.startswith('alt_') or force_alt:
+                self.assertNotIn(f'name - {reduced_inst_name}', doc_string,
+                                 f'name - {reduced_inst_name} found in {node.full_inst_name}')
+                self.assertNotIn(f'desc - {reduced_inst_name}', doc_string,
+                                 f'desc - {reduced_inst_name} found in {node.full_inst_name}')
+
+                if 'field_a' not in node.inst_name:
+                    for child_node in node:
+                        check_item(child_node, True)
+
+            else:
+                self.assertIn(f'name - {reduced_inst_name}', doc_string,
+                              f'name - {reduced_inst_name} not found in {node.full_inst_name}')
+                self.assertIn(f'desc - {reduced_inst_name}', doc_string,
+                              f'desc - {reduced_inst_name} not found in {node.full_inst_name}')
+
+                if 'field_a' not in node.inst_name:
+                    for child_node in node:
+                        check_item(child_node)
+
+        with self.build_python_wrappers_and_make_instance(skip_name_desc_in_docstring=False) as dut:
+            for child_node in dut:
+                check_item(child_node)
+
+    if sys.version_info < (3, 14):
+        # python 3.14 introduced assertIsSubclass, it was not present before then so we have to
+        # use a manual version
+
+        # pylint: disable=missing-function-docstring,invalid-name
+
+        def assertIsSubclass(self, cls, class_or_tuple, msg=None):
+            if not issubclass(cls, class_or_tuple):
+                standard_msg = f"{cls.__name__} is not a subclass of {class_or_tuple}"
+                self.fail(self._formatMessage(msg, standard_msg))
+
+        def assertNotIsSubclass(self, cls, class_or_tuple, msg=None):
+            if issubclass(cls, class_or_tuple):
+                standard_msg = f"{cls.__name__} is a subclass of {class_or_tuple}"
+                self.fail(self._formatMessage(msg, standard_msg))
+
+    def test_skipped(self):
+        """
+        Test to make sure all the nodes have a name and description excluded from the docstring
+        """
+        def check_item(node: Node, type_dict:dict[str, list[type[Node]]]):
+            doc_string = inspect.getdoc(node)
+            if node.inst_name.endswith('[0]') or node.inst_name.endswith('[1]'):
+                reduced_inst_name = node.inst_name[:-3]
+            else:
+                reduced_inst_name = node.inst_name
+            self.assertNotIn(f'name - {reduced_inst_name}', doc_string,
+                             f'name - {reduced_inst_name} found in {node.full_inst_name}')
+            self.assertNotIn(f'desc - {reduced_inst_name}', doc_string,
+                             f'desc - {reduced_inst_name} found in {node.full_inst_name}')
+
+            for inst_name_pattern, list_of_types in type_dict.items():
+                if inst_name_pattern in node.inst_name:
+                    if type(node) not in list_of_types:
+                        list_of_types.append(type(node))
+
+            if 'field_a' not in node.inst_name:
+                for child_node in node:
+                    check_item(child_node, type_dict)
+
+        with self.build_python_wrappers_and_make_instance(skip_name_desc_in_docstring=True) as dut:
+            type_dict={
+                'field_a':[],
+                'reg_a':[],
+                'reg_b':[],
+                'regfile_a':[],
+                'regfile_b': [],
+                'addrmap_a': [],
+                'addrmap_b': [],
+                'mem_a': [],
+                'mem_b': [],
+            }
+            for child_node in dut:
+                check_item(child_node, type_dict)
+
+            # with the docstring suppressed, the _alt versions should map to the same classes
+            self.assertEqual(len(type_dict['field_a']), 1)
+            self.assertIs(type_dict['field_a'][0], FieldReadWrite)
+            for inst_name_pattern, types in filter(lambda kv: kv[0]!='field_a',type_dict.items()):
+                if inst_name_pattern.endswith('_a'):
+                    # all the xxxx_a should be a single instance
+                    self.assertEqual(len(types), 1)
+                else:
+                    # all the xxxx_b should be an node_array and a node of the respective types
+                    self.assertIsSubclass(types[0], NodeArray)
+                    self.assertNotIsSubclass(types[1], NodeArray)
+
+
 
 if __name__ == '__main__':
 
