@@ -1,4 +1,20 @@
 """
+peakrdl-python is a tool to generate Python Register Access Layer (RAL) from SystemRDL
+Copyright (C) 2021 - 2025
+
+This program is free software: you can redistribute it and/or modify
+it under the terms of the GNU Lesser General Public License as 
+published by the Free Software Foundation, either version 3 of 
+the License, or (at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU Lesser General Public License for more details.
+
+You should have received a copy of the GNU Lesser General Public License
+along with this program.  If not, see <https://www.gnu.org/licenses/>.
+
 Tests for the array indexing in the base library
 """
 
@@ -67,7 +83,7 @@ class ArrayBase(CallBackTestWrapper, ABC):
 
         class DUTWrapper(AddressMap):
             """
-            Address map to to wrap the register array being tested
+            Address map to wrap the register array being tested
             """
 
             # pylint: disable=too-many-arguments,duplicate-code
@@ -87,26 +103,11 @@ class ArrayBase(CallBackTestWrapper, ABC):
                                                inst_name='dut',
                                                parent=self,
                                                address=address,
-                                               accesswidth=32,
-                                               width=32,
                                                stride=dut_stride,
                                                dimensions=dut_dimensions)
 
-            def get_memories(self, unroll: bool = False) -> \
-                    Iterator[Union[Memory, tuple[Memory, ...]]]:
-                raise NotImplementedError('Not implemented in the testing')
-
-            def get_sections(self, unroll: bool = False) -> \
-                    Iterator[Union[Union[AddressMap, RegFile],
-                                   tuple[Union[AddressMap, RegFile], ...]]]:
-                raise NotImplementedError('Not implemented in the testing')
-
-            def get_registers(self, unroll: bool = False) -> \
-                    Iterator[Union[Reg, RegArray]]:
-                """
-                generator that produces all the readable_registers of this node
-                """
-                raise NotImplementedError('Not implemented in the testing')
+            def __iter__(self) ->  Iterator[Union[Node, NodeArray]]:
+                yield self.dut
 
             @property
             def systemrdl_python_child_name_map(self) -> dict[str, str]:
